import 'dart:io';
import 'dart:ui';

import 'package:camera/camera.dart';
import 'package:camera_platform_interface/camera_platform_interface.dart';
import 'package:device_info/device_info.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:image/image.dart' as imglib;
import 'package:kyc/Components/Navigator/back_pop_up.dart';
import 'package:kyc/Components/Preview/document_preview.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Components/state.dart';
import 'package:kyc/Models/kyc_document.dart';
import 'package:path_provider/path_provider.dart';
import 'package:provider/provider.dart';

/*
TODO
 EXPLODE THE HUGE COMPONENT INTO SMALLER ONES
 FORCE PORTRAIT CAMERA
 ADD EDGE DETECTION
 TRIGGER PICTURE TAKING ON EDGE DETECTION
 HANDLE SWITCH APP FOCUS
*/
class FaceScanScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Stack(
      fit: StackFit.expand,
      children: [
        Align(
            alignment: Alignment.topCenter,
            child: FaceScanStep(title: "Face Scan"))
      ],
    ));
  }
}

class FaceScanStep extends StatefulWidget {
  final String title;

  FaceScanStep({key, this.title}) : super(key: key);

  @override
  _FaceScanStepState createState() => _FaceScanStepState();
}

class _FaceScanStepState extends State<FaceScanStep> {
  bool _cameraInit = false;
  bool _isPhysicalDevice = false;
  CameraController _cameraController;

  @override
  initState() {
    super.initState();
    if (Platform.isAndroid || Platform.isIOS) {
      _initCamera();
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    final CameraController cameraController = _cameraController;

    // App state changed before we got the chance to initialize.
    if (cameraController == null || !cameraController.value.isInitialized) {
      return;
    }

    if (state == AppLifecycleState.inactive) {
      cameraController.dispose();
    } else if (state == AppLifecycleState.resumed) {
      cameraController.addListener(() {
        if (mounted) setState(() {});
      });
    }
  }

  _initCamera() async {
    if (Platform.isMacOS || Platform.isWindows) {
      _isPhysicalDevice = false;
    } else if (Platform.isIOS) {
      _isPhysicalDevice = (await DeviceInfoPlugin().iosInfo).isPhysicalDevice;
    } else if (Platform.isAndroid) {
      _isPhysicalDevice =
          (await DeviceInfoPlugin().androidInfo).isPhysicalDevice;
    }
    if (!_isPhysicalDevice) {
      _cameraInit = true;
      return;
    }
    try {
      List<CameraDescription> cameras = await availableCameras();
      CameraDescription frontCamera = cameras.firstWhere(
          (element) => element.lensDirection == CameraLensDirection.front);
      final CameraController cameraController = CameraController(
        frontCamera,
        ResolutionPreset.low,
        enableAudio: false,
      );
      cameraController.addListener(() {
        if (mounted) setState(() {});
      });
      _cameraController = cameraController;
      await _cameraController.initialize();
      await _cameraController
          .lockCaptureOrientation(DeviceOrientation.portraitUp);
      _cameraInit = true;
    } on CameraException catch (e) {
      print('Error: ${e.code}\nError Message: ${e.description}');
    }
  }

  @override
  dispose() {
    super.dispose();
    if (_cameraController != null) {
      _cameraController.dispose();
    }
  }

  _takePicture(BuildContext context) async {
    final CameraController cameraController = _cameraController;
    Image img;
    File croppedFile;
    XFile imageFile = await cameraController.takePicture();
    imglib.Image picture = imglib.decodeJpg(await imageFile.readAsBytes());
    picture = imglib.flipHorizontal(picture);
    int width = picture.width;
    int height = picture.height;
    int sightHeight = (height * 0.8).toInt();
    int sightWidth = (width * 0.55).toInt();
    var sideOffset = (width * 0.2).toInt();
    var topOffset = (height * 0.1).toInt();
    imglib.Image croppedPicture = imglib.copyCrop(
        picture, sideOffset, topOffset, sightWidth, sightHeight);
    Directory dir = await getApplicationSupportDirectory();
    String fileName = dir.path + "/" + DateTime.now().toString() + "face.jpg";
    var jpgPicture = imglib.encodeJpg(croppedPicture);
    croppedFile = File(fileName);
    croppedFile = await croppedFile.writeAsBytes(jpgPicture);
    img = Image.memory(jpgPicture, fit: BoxFit.fitHeight);
    _redirect(context, img, croppedFile.path);
  }

  void _redirectSimulator(BuildContext context) {
    Widget redirection = context.read<KycState>().getFaceSimulatorPreview();
    String path = context.read<KycState>().getSimulatorFacePreviewPath();

    this._redirect(context, redirection, path);
  }

  void _redirect(BuildContext context, Image img, String imagePath) {
    context.read<KycState>().completeProcess();
    context.read<KycState>().document = new KycDocument();
    context.read<KycState>().setDocumentType(DocumentType.FaceScan);
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) =>
                DocumentPreview(img: img, imgPath: imagePath)));
  }

  _buildTakePictureButton(BuildContext context) {
    return FloatingActionButton(
        child: Icon(Icons.photo_camera_rounded),
        onPressed: () => _isPhysicalDevice
            ? _takePicture(context)
            : _redirectSimulator(context));
  }

  @override
  Widget build(BuildContext context) {
    String instruction = "Selfie Step";
    String inscructionStep = "Position your face in the sight";

    double height = MediaQuery.of(context).size.height;
    double width = MediaQuery.of(context).size.width;
    var preview;
    if (!context.read<KycState>().isPhysicalDevice()) {
      _cameraInit = true;
      preview = context.read<KycState>().getFaceSimulatorPreview();
    } else {
      preview = CameraPreview(_cameraController);
    }
    var sideOffset = (width * 0.05);
    return WillPopScope(
      onWillPop: () {
        BackPopup.standardPopup(context);
        return;
      },
      child: Scaffold(
        appBar: AppBar(title: Text("Face Scanner")),
        body: Center(
            child: !_cameraInit
                ? CircularProgressIndicator()
                : SizedBox.expand(
                    child: Stack(
                      fit: StackFit.expand,
                      children: [
                        preview,
                        ColorFiltered(
                            colorFilter: ColorFilter.mode(
                                Colors.black.withOpacity(0.8),
                                BlendMode.srcOut),
                            child: Stack(
                              fit: StackFit.expand,
                              children: [
                                Container(
                                  decoration: BoxDecoration(
                                      color: Colors.black,
                                      backgroundBlendMode: BlendMode.dstOut),
                                ),
                                Positioned(
                                    top: height*0.1,
                                    left: width*0.15,
                                    child: ClipOval(
                                        child: Container(
                                            width: width * 0.7,
                                            height: width - sideOffset,
                                            decoration: BoxDecoration(
                                              color: Colors.black,
                                            )))),
                              ],
                            )),
                        //TODO CONTAINER FOR CORNER PICTURE SIGHT
                        Positioned(
                            width: width,
                            bottom: height * 0.15,
                            child: Center(
                                child: Text(
                              inscructionStep,
                              style:
                                  TextStyle(fontSize: 25, color: Colors.white),
                              textWidthBasis: TextWidthBasis.parent,
                            ))),
                        Positioned(
                            width: width,
                            bottom: height * 0.13,
                            child: Center(
                                child: Text(
                              instruction,
                              style:
                                  TextStyle(fontSize: 15, color: Colors.white),
                              textWidthBasis: TextWidthBasis.parent,
                            ))),
                      ],
                    ),
                  )),
        floatingActionButton: _buildTakePictureButton(context),
        floatingActionButtonLocation: FloatingActionButtonLocation.centerDocked,
      ),
    );
  }
}
